#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QWidget>
#include <QObject>
#include <QSpinBox>
#include <QComboBox>
#include <QLabel>
#include <QTimer>
#include <QLCDNumber>
#include <QString>
#include <QSignalMapper>
#include <QPushButton>

#include <math.h>

#include "common.h"
#include "lcdrange.h"
#include "landscapearea.h"
#include "gaga.h"

class MainWindow : public QWidget
{
    Q_OBJECT

public:
   MainWindow(QWidget *parent = 0);
   ~MainWindow();

public slots:
   void slot_RunState(int steps);
   void setFitness(int fitness);
   void setAmplitude(int amplitude);
   void setMuteProb(int muteprob);
   void setMuteBias(int mutebias);

   void slot_gaga(void);
   void slot_Reload(void);
   void slot_RunPreState(void);
   void slot_StepPreState(void);
   void slot_StopState(void);
   void slot_NewGenes(void);
   void slot_LoadFile(void) ; // QString);
   void slot_LShift(void);
   void slot_RShift(void);
   void slot_UShift(void);
   void slot_DShift(void);

signals:
   void FitnessChanged(int) ;
   void AmplitudeChanged(int) ;
   void MuteProbChanged(int) ;
   void MuteBiasChanged(int) ;

   void signal_LoadFile(void) ; // QString) ;
   void signal_RunPreState(int) ;
   void signal_StopState(void) ;
   void signal_EnvironmentUpdate(void);
   void signal_LandscapeUpdate(void);
   void signal_MMRsUpdate(void);
   void signal_GenomeUpdate(void);

private:
   QTimer* gaga_timer_ptr ;

   LandscapeArea* Lbox_ptr ;

   QComboBox*     InputFileComboBox_ptr ;

   QPushButton*   runBTN_ptr ;
   QPushButton*   stepBTN_ptr ;
   QPushButton*   stopBTN_ptr ;

   QPushButton*   loadBTN_ptr ;
   QPushButton*   newgenesBTN_ptr ;

   QLabel*        integerLabelParam0_ptr ;
   QLabel*        integerLabelParam1_ptr ;
   QLabel*        integerLabelParam2_ptr ;
   QLabel*        integerLabelParam3_ptr ;

   QSpinBox*      integerSpinBoxParam0_ptr ;
   QSpinBox*      integerSpinBoxParam1_ptr ;
   QSpinBox*      integerSpinBoxParam2_ptr ;
   QSpinBox*      integerSpinBoxParam3_ptr ;

   
   QString ttip_clearhistBTN ;
   QString ttip_fitnessBOX ;
   QString ttip_mutebiasBOX ;
   QString ttip_muteprobBOX ;

   QLCDNumber* generationLCD_ptr ;

   int freeToRun ;
   int MuteProb ;
   int bit ;
   int membr ;
   int theRefEnvironment[ENVIRONMENT_LENGTH] ;
   int theEnvironment[ENVIRONMENT_LENGTH] ;
   int theCommunity[MEMBERS][GENOME_BITS] ;

   // FIXME: iterations is here just in case the Stop Button is unreachable
   // due to the program hogging resources.
   // Reduced to a flag, finish.
   int iterations ;


   int generations ;

   int currentFitness ;
   int currentAmplitude ;
   int currentMuteProb ;
   int currentMuteBias ;

   int Param0 ;
   int Param1 ;
   int Param2 ;
   int Param3 ;


};
#endif //MAINWINDOW_H

