#include "gaga.h"
#include "common.h"

using namespace std;

/* This program is accepts
   -- a collection (members) of bit vectors (genome),
   -- an array of integers (environment),
   -- and some other parameters,
   makes changes to the genome, and returns. It is the change that matters.

   Each genome is an unsigned integer, and a genetic algorithm
   is used to guide changes to those bit vectors.

   These bit vectors become indexes to the input array of integers, the environment. Changes
   are made such that the integer representations of each bit vector indicate a particular
   point in that environment and, iteration by iteration, they all converge on the maximum
   value in the environment array.

   The array of integers is an "environment":
   int my_environment[]     The array of integers

   The collection of bitvectors is a "community":
   int my_community[][]     The collection of bitvectors

 */

//-------------------------------------------------------------
// copy a member, from->to. Just like is says on the box
void copy_mbr(int* to, int* from)
{
   for (int bit=0 ; bit<GENOME_BITS ; bit++) { to[bit] = from[bit] ; }
}

//-------------------------------------------------------------
//
int gaga(
/* This function does not currently return any value of interest, so the
 * non void return type is a placeholder. Perhaps the next version will
 * return, for example, some metric about the degree of convergence or
 * similarity among the members' genomes. But all this information would 
 * also be available to the calling function and only the calling function
 * would know such things as how many iterations gaga ran.
 * Stopping is not a concern of gaga.
 */

      int mutation_probability,
/*   O is 0% probability that a bit will randomly change, and
 * 100 is 100% probability.
 */

      int mutation_bias,
/*  O means that this parameter is having no effect.
 * 10 means that the mutation_probability is heavily biased towards small
 * changes. 
 *   So the probability of mutation of bit 0 mutating is 2 times higher
 * than the probability of mutation of bit 1 mutating is 2 times higher
 * than the probability of mutation of bit 2 mutating is 2 times higher
 * than the probability of mutation of bit 3 mutating is 2 times higher
 * etc.
 */

      int fitness_multiplier,
/* Please see the code for this, it is just a multiplier.
 * HOWEVER, abusing this multiplier is the quickest way to send this code into
 * arithmetic error. 
 */

      int my_environment[ENVIRONMENT_LENGTH],
/* The environment is just an array of 512 ints. In this demonstration
 * program, gaga cannot modify the environment.
 *        (That implies that the best resources of the
 *         environment are infinitely exploitable and 
 *         apparently automatically renewed. It would
 *         be interesting to modify this to a 
 *         function that degrades the environment even
 *         as it seeks to find the best part of it.
 *         It would be more human that way.)
 * Because gaga is passed a pointer to this array, this means that
 * changing the environment while gaga is running will affect the function.
 * Please see the next comment for an explanation as to why I don't care
 * about this.
 */

      int my_community[][GENOME_BITS],
/* The community is a MEMBERS length array of GENOME_BITS of integers.
 * The result of gaga running, its function, is to return the community
 * in a more fit condition than that in which it was received. It will
 * do this by directly modifying the genomes of each of the members of
 * the community.
 * Yes this does imply that the community data structure and the environment
 * can be written to and possibly "corrupted" while gaga is running. But I
 * have implemented a mutex elsewhere, yes? No. So there is a semaphore for
 * access to these, yes? No. The reason for not including a locking
 * mechanism of some sort on the shared data is that
 * 1) the only preempting that can occur is the result of a user button
 *    click, and 
 * 2) the only buttons the user can click while gaga is free running are
 *    functions which modify essentially a guided random process, a process
 *    that will run hundreds of cycles before you get your finger off that
 *    button. 
 * Waiting for gaga to complete, holding it off, modifying the data, and
 * restarting gaga will indeed produce different results than just changing
 * the data at will. But for the purpose of this demonstration program it
 * will have no discernable effect.
 */

      int param0,
      int param1,
      int param2,
      int param3
/* These are placeholders. There are matching placeholders on the GUI panel 
 * so that you can add some functionality to gaga and stll use the GUI as-is.
 */
)

{  //body of gaga starts here.
   int membr ;
   static int       mutation_biases[GENOME_BITS] ; 
   static int      my_mbrs[MEMBERS][GENOME_BITS] ; // internal copy
   static int   my_mbrsNew[MEMBERS][GENOME_BITS] ; // internal copy
   static int   my_mbrsInt[MEMBERS] ; // The positive integer value of the genome
   static int      fitness[MEMBERS] ; 
   int* ww ;    // The Weighted Wheel
   int fitscan ;

   static int total_mutebias = 0 ; 
   static int my_mutation_bias = 11 ; // trick: init to > 10 to force initialization.
   static int beentheredonethat = 0 ;

   static int                bstats[GENOME_BITS] ; 

   // initializations
   if (beentheredonethat == 0)
   {
      srand(RANDSEED) ;
      for (int g=0 ; g<GENOME_BITS ; g++) { bstats[g]=0 ; }
      beentheredonethat = 1 ;
   }

   if (mutation_bias != my_mutation_bias)
   {
      total_mutebias = 0 ;
      my_mutation_bias = mutation_bias ;
      for (int g=0;g<GENOME_BITS;g++)
      {
         mutation_biases[g] = (int)(power((my_mutation_bias*(GENOME_BITS-g)),2)) ;
         total_mutebias += mutation_biases[g] ;
      }
   }

   // And the Genetic Algorithm is ...
   //     read each member's genes,
   //         (calculate their fitnesses) make this a separate function,
   //             make the weighted wheel,
   //                 generate breeding pairs,
   //                     mix genes,
   //                         (maybe random flip) make the decision a separate function,
   //                             write genes back (replace everybody)
   //


   // save a copy of each mbr's genome for manupulation,
   // and convert to a single int for fitness checking.
   for (membr=0;membr<MEMBERS;membr++)
   {
      copy_mbr(my_mbrs[membr], my_community[membr]) ;
      my_mbrsInt[membr] = genomeAsInteger(GENOME_BITS, my_community[membr]) ;
   }

   // calculate the individual fitnesses and the total fitness
   int total_fitness = 0 ;
   for (membr=0;membr<MEMBERS;membr++)
   {
      fitness[membr] = my_environment[my_mbrsInt[membr]] ;
      total_fitness = total_fitness + fitness[membr] ;
   }
   if (total_fitness == 0) {
      // Then I'm about to divide by zero.
      cout << "total_fitness = 0. Not good. Seting to 1.\n" ;
      total_fitness = 1 ;
   }
   // Find the lowest fitness
   int lowest = total_fitness ;
   for (membr=0;membr<MEMBERS;membr++)
   {
      if (fitness[membr] < lowest)
      {
         lowest = fitness[membr] ;
      }
   }
   lowest-- ;
   // Subtract the lowest , but don't go negative
   for (membr=0;membr<MEMBERS;membr++)
   {
      fitness[membr] = fitness[membr] - lowest ;
      fitness[membr] = fitness_multiplier * fitness[membr] ;
   }
   // do it over again. now scaled
   total_fitness = 0 ;
   for (membr=0;membr<MEMBERS;membr++)
   {
      total_fitness = total_fitness + fitness[membr] ;
   }

   // make weighted wheel,
   // This weighted wheel implementation is the first pass and is
   // rather barbaric, time consuming, and has the greatest potential for
   // numeric overflow. It will be replaced soon. 
   /* The wheel looks like an array of
    * fitness[0] number of zeros,
    * fitness[1] number of ones,
    * ...
    * fitness[MEMBERS-1] number of MEMBERS-1
    * The higher the fitness of a member, the more numbers that member
    * has, or the more space it takes up on the wheel. Choosing a number
    * at random from the wheel will most probably choose the member with
    * the higher fitness.
    */
   fitscan = 0 ;
   ww = (int*) (malloc(total_fitness*sizeof(int))) ;
   for (membr=0;membr<MEMBERS;membr++)
   {
      for (int fit=0;fit<fitness[membr];fit++)
      {
         ww[fitscan] = membr ;
         fitscan++ ;
      }
   }

   // generate breeding pairs,
   // This is done on probability, generate MEMBERS number of ints, ints 
   // which *are* the indexes for which pairs to breed.
   //
   int breeding_units[MEMBERS] ; // These are indexes to the members which will be bred
   int a_random_num ;
   for (membr=0;membr<MEMBERS;membr++)
   {
      a_random_num = rand() % total_fitness ; // Choose some point in the total fitness.
      breeding_units[membr] = ww[a_random_num] ;
   }
   free (ww) ;

   // mix genes,
   //
   // my_mbrs provide data for breeding and the new breeds go into my_mbrsNew
   // and when the breeding is done all creatures are updated with the my_mbrsNew
   for (membr=0;membr<MEMBERS;membr++)
   {
      copy_mbr(my_mbrsNew[membr], my_mbrs[membr]) ;
   }

   for (membr=0;membr<MEMBERS;membr++)
   {
      // Choose some point to cross/split gene
      //
      a_random_num = rand() % GENOME_BITS ;
      copy_mbr(my_mbrsNew[membr],   my_mbrs[breeding_units[membr]]) ;
      copy_mbr(my_mbrsNew[membr+1], my_mbrs[breeding_units[membr+1]]) ;

      for (int split = 0;split < a_random_num; split++)
      {
         my_mbrsNew[membr][split]   = my_mbrs[breeding_units[membr+1]][split] ;
         my_mbrsNew[membr+1][split] = my_mbrs[breeding_units[membr]][split] ;
      }
      membr++ ;
   }


   // maybe random bit flips,
   //
   int b_random_num ;
   for (membr=0;membr<MEMBERS;membr++)
   {
      if (mutation_probability != 0) {
         a_random_num = rand() % (100-mutation_probability) ;
         if (a_random_num == 0)
         {
            if (my_mutation_bias != 0)
            {
               b_random_num = 1+rand() % total_mutebias ;
               int g = 0 ;
               int acc = mutation_biases[0] ;
               while ((b_random_num > acc) && (g < GENOME_BITS))
               {
                  g++ ;
                  acc += mutation_biases[g] ;
               }
               b_random_num = g ;
            }
            else
            {
               b_random_num = rand() % (GENOME_BITS) ;
            }
            // This "1 if 0, 0 if 1" code could be simpler but it is left
            // as a placeholder (I was considering using the remaining bits
            // of each int for statistics rather than just using an int
            // as only a 1 or 0.
            if (my_mbrsNew[membr][b_random_num] == 0) {
                my_mbrsNew[membr][b_random_num] = 1 ;
            } else {
                my_mbrsNew[membr][b_random_num] = 0 ;
            }
         }
      }
   }

   // write genes back
   //
   for (membr=0 ; membr<MEMBERS ; membr++)
   {
      copy_mbr(my_community[membr], my_mbrsNew[membr]) ;
   }

   return 0 ;
}

