#ifndef GAGA_H
#define GAGA_H

#include "common.h"

int gaga (
   int mutation_probability,
   /*   O is 0% probability that a bit will randomly change, and
    * 100 is 100% probability.
    */

   int mutation_bias,
   /*  O means that this parameter is having no effect.
    * 10 means that the mutation_probability is heavily biased towards small
    * changes. 
    *   So the probability of mutation of bit 0 mutating is 2 times higher
    * than the probability of mutation of bit 1 mutating is 2 times higher
    * than the probability of mutation of bit 2 mutating is 2 times higher
    * than the probability of mutation of bit 3 mutating is 2 times higher
    * etc.
    */

   int fitness_multiplier,
   /* Please see the code for this, it is just a multiplier.
    * HOWEVER, abusing this multiplier is the quickest way to send this
    * code into an arithmetic error. 
    */

   int my_environment[],
   /* The environment is just an array of 512 ints. In this demonstration
    * program, gaga cannot modify the environment.
    *        (That implies that the best resources of the
    *         environment are infinitely exploitable and 
    *         apparently automatically renewed. It would
    *         be interesting to modify this to a 
    *         function that degrades the environment even
    *         as it seeks to find the best part of it.
    *         It would be more human that way.)
    * Because gaga is passed a pointer to this array, this means that
    * changing the environment while gaga is running will affect the function
    * in an unpredictable way.
    * Please see the next comment for an explanation as to why I don't care
    * about this.
    */

   int my_community[][GENOME_BITS],
   /* The community is a MEMBERS length array of GENOME_BITS of integers.
    * The result of gaga running, its function, is to return the community
    * in a more fit condition than that in which it was received. It will
    * do this by directly modifying the genomes of each of the members of
    * the community.
    * Yes this does imply that the community data structure and the landscape
    * can be written to and possibly "corrupted" while gaga is running. But I
    * have implemented a mutex elsewhere, yes? No. So there is a semaphore for
    * access to these, yes? No. The reason for not including a locking
    * mechanism of some sort on the shared data is that
    * 1) the only preempting that can occur is the result of a user button
    *    click, and 
    * 2) the only buttons the user can click while gaga is free running are
    *    functions which modify essentially a guided random process, a process
    *    that will run hundreds of cycles before you get your finger off that
    *    button. 
    * Waiting for gaga to complete, holding it off, modifying the data, and
    * restarting gaga will indeed produce different results than just changing
    * the data at will. But for the purpose of this demonstration program it
    * will have no discernable effect.
    */
   int param0,
   int param1,
   int param2,
   int param3
   /* These are placeholders. There are matching placeholders on the GUI panel 
    * so that you can add some functionality to gaga and stll use the GUI as-is.
    */
) ;

#endif // GAGA_H
